//ESP8266_SetLED Version1 17.01.2015 von Fabian Kainka

#include "SoftwareSerial.h"

#define xSSID "Hier könnte Ihre SSID stehen"
#define xPASSWORT "Hier könnte Ihr Passwort stehen"
#define xPORT 80

#define LED 13

SoftwareSerial ESPSerial(10, 11); // RX, TX

boolean ESP_Aktiv;
boolean WLAN_Connected;
boolean SERVER_Aktiv;

boolean NC; //zustand mögliches neues Kommando wegen NewLine


void setup() {
  pinMode(LED, OUTPUT);

  Serial.begin(9600);
  ESPSerial.begin(9600);
  ESPSerial.setTimeout(5000);


  while (!Serial) {
    ; // wait for serial port to connect. Needed for Leonardo only
  }
  debug("Debug Serial OK");


  //einfache AT frage ob Modul OK
  ESP_Aktiv = ESP_Test();

  //Modul einstellen
  if (ESP_Aktiv) {
    //Verhindert das jedesmal alles Resetet wird beim Programmieren oder reseten nur des Controllers
    int Status = SERVER_Status();
    debug("ServerStatus:" + String(Status));
    //4 oder 5 heißt wohl server down / wlan nicht verbunden
    if (Status == 4 || Status == 5) {
      ESP_TotalStart();
    }
    else {
      SERVER_Aktiv = true;
      WLAN_Connected = true;
    }
  }
}


void loop() {

  while (ESPSerial.available()) {
    char inChar = ESPSerial.read();

    if (NC == true) //Nur wenn NL das letze Zeichen war wird dieses Zeichen auf +,r, L oder U untersucht!! dadruch beschleunigt //Wenn hier false in die If-Afrage einteragen wird, gehjt alles in Debug durch- keine interpretation
    {
      if (inChar  == '+' || inChar  == 'r' || inChar  == 'L' || inChar  == 'U') //Diese 4 sind der beginn eines evtl interessanten Kommandos
      {
        switch (inChar) {
          case '+':
            {
              //debug("Befehl/Request?");
              delay (1); //Delay scheont die Zuverlässigkeit von der Befehlserkennung zu erhhöhen
              char NextPeek = ESPSerial.peek();
              if (NextPeek == 'C') {
                break; //z.B. durch CWLAP möglichst schneller austieg damit keine DAten aus Buffer verloren gehen
              }

              if (NextPeek == 'I') {//Vermutung IPD = Einkommender Befehl
                if (ESPSerial.findUntil("IPD", " ")) { //Bestätigen
                  debug("IPD-Kommando");
                  SERVER_IPD(); //Unterfunktion zum analysieren des IPD-Kommandos
                  inChar = 0;
                }
              }
              break;
            }

          case 'r':  //ready vom Modul gesendet=?
            //debug("Ungeplanter Neustart??");
            if (ESPSerial.findUntil("eady", "/n")) {
              debug("Ungeplanter ModulReset -> Neustart");
              inChar = 0;
              ESP_TotalStart(); //Aufruf der Startroutine
            }
            break;

          case 'L':
            //debug("Link? ");  //Linked: eine Verbindung wurde aufgebaut... Evtl für später zu gerbauchen -> senden an Clients
            if (ESPSerial.findUntil("ink", " ")) {
              debug("Linked!");
              inChar = 0;
            }
            break;

          case 'U':  //UnLinked: eine Verbindung wurde getrennt... Evtl für später zu gerbauchen -> senden an Clients //ALLERDINGS: scheint selten ausgeläst zu werden
            debug("Unlink?"); //Hier kommen Daten gelegentlich weg die mit U beginnen in neuer Zeile ---  kommt eher bei Browser request vor.. sowieso abfangen in request befehl
            if (ESPSerial.findUntil("nlink", " ")) {
              debug("Unlinked");
              inChar = 0;
            }
            break;
        }
      }
    }
    //New Line ermöglicht einkommenden Befehl // damit beschleunigt
    if (inChar == '\n') {
      NC = true;
    }    else {
      NC = false;
    }

    //Debug-> alles lässt sich über die Konsole verfolgen
    Serial.write(inChar);
  }
}


//  --------------------Funktionen--------------------------------------

//--------------------Analyse von +IPD Befehlen---------------------------------
void SERVER_IPD() {
  //Beispiel +IPD,0,376:GET / HTTP/1.1 = Typischer Browser request
  int Client;    //Client ID
  int NZeichen;   //Anzahl der gesendeten Zeichen

  char buffer[5];    //Buffer eig nur für Müll
  char Kommando1[10]; //Buffer für das erste Kommando direkt nach dem : bis Leerzeichen
  char Kommando2[10];  //Buffer für zweites Kommando... noch nicht ausgereift

  //Löscht alles aus Kommando1 und 2- Scheint wichtig sonst bleibt müll...memset schnellste möglichkeit
  memset(Kommando1, 0, sizeof(Kommando1));
  memset(Kommando2, 0, sizeof(Kommando2));

  Client = ESPSerial.parseInt();
  NZeichen = ESPSerial.parseInt();

  ESPSerial.readBytesUntil(':', buffer, 5);
  ESPSerial.readBytesUntil(' ', Kommando1, 10); //hier konvention für Kommandolänge oder Ähnliches


  //Vorrausslich Request über Server
  if (strcmp (Kommando1, "GET") == 0) {
    ESPSerial.readBytesUntil(' ', Kommando2, 10);
    debug ("kommando2: " + String(Kommando2));

    //Das aktivierende Kommando ist im browser IP:80/Led1 - Alles andere deaktivert die LED
    if (strcmp (Kommando2, "/Led1") == 0) {
      digitalWrite (LED, HIGH);
      debug("LED AN");
    }
    else {
      digitalWrite (LED, LOW);
      debug("LED AUS");
    }

    if ( ESPSerial.find("OK")) {
      debug ("OK Skipt"); //hier wird der ganze Quatsch vom Browser Request übersprungen... Dauet ne weile
    }
  }


  //Vorrausslich Request über TCP-Cient
  else if (strcmp (Kommando1, "set") == 0) {
    debug("set");
    //kommando über TCP client: set 1 oder set 0
    digitalWrite(13, ESPSerial.parseInt());
  }

  //Ausgabe zu debug zwecken
  debug("von Client: " + String(Client) + " Zeichenanzahl: " + String(NZeichen) + " Kommando1: " + Kommando1); //+ " Kommando2: "+ Kommando2
}


//--------------------Starten des Moduls: Einstellung WLAN und Server--------------------------------
void ESP_TotalStart() {
  //Reset zur sicherheit
  ESP_Reset();

  ESP_Aktiv = ESP_Test();
  if (ESP_Aktiv == true) {
    // WLAN verbinden
    WLAN_Connected = WLAN_Connect(xSSID, xPASSWORT);
  }
  if (WLAN_Connected == true) {
    //wenn Wlan verbunden Server Starten
    SERVER_Aktiv = SERVER_Start(80);
  }
}


//--------------------Einfache Frage Modul AT Antwort OK --------------------------------
boolean ESP_Test() {
  ESPSerial.println("AT");

  if (ESPSerial.find("OK"))
  {
    debug("Module OK");
    return true;
  }
  else
  {
    debug("Modul Fehler!");
    return false;
  }
}


//--------------------Modul Reseten --------------------------------
boolean ESP_Reset() {
  ESPSerial.println("AT+RST");
  if (ESPSerial.find("ready")) {
    debug ("RESET OK");
    return true;
  }
  else {
    debug ("RESET ERROR");
    return false;
  }

}


//--------------------WLAN Verbinden --------------------------------
boolean WLAN_Connect(char SSID[], char  PASSWORT[] ) {
  //Einstellung für Datenmodus : 1 = Datenmodus; 0 = Transparent
  ESPSerial.println("AT+CWMODE=1");

  //Abfrage ob WLAN schon verbunden
  ESPSerial.println("AT+CWJAP?");

  //Schnellausstief aus Funktion wenn schon mit besagtem Netzwerk verbunden
  // beschleundig durch error ausstieg
  if ( ESPSerial.findUntil(SSID, "ERROR")) {
    debug("WLAN already connected");
    return true;
  }

  //String Zusammenfassung für WLAN Verbinden
  String cmd = "AT+CWJAP=\"";
  cmd += SSID;
  cmd += "\",\"";
  cmd += PASSWORT;
  cmd += "\"";
  ESPSerial.println(cmd);

  //Wartezeit weil verbindung lange braucht
  delay(3000);
  if (ESPSerial.find("OK"))
  {
    Serial.println("WLAN Connected");
    return true;
  }
  else
  {
    Serial.println("WLAN ERROR");
    return false;
  }
}


//--------------------Server Status --------------------------------
int SERVER_Status() {
  ESPSerial.println("AT+CIPSTATUS");
  return ESPSerial.parseInt();
}


//--------------------Server Starten --------------------------------
boolean SERVER_Start(int Port) {
  //Mehrere Verbidnungen zulassen, wichtig sonst startet Server nicht
  ESPSerial.println("AT+CIPMUX=1");

  if (ESPSerial.find("OK"))
  {
    String cmd = " AT+CIPSERVER=1,";
    cmd += Port;

    ESPSerial.println(cmd);
    //delay (2000); Vermute Wartezeit unnötig
    if (ESPSerial.find("OK")) {
      debug ("Server OK");
      return true;
    }
    else {
      debug ("Server ERROR");
      return false;
    }
  }
  else {
    debug ("Error AT+CIPMUX");
    return false;
  }
}



//_____________________________DEBUG FUNKTIONEN über SERIAL_____________________________
void debug (String Text) {
  Serial.println(Text);
}

void serialEvent() {
  //alles was über Terminal gesendet wird ans Modul weiter senden --- Funktion serialEvent wird automatisch aufgerufen wenn Daten über die Hardwareserielle reinkommen
  ESPSerial.write(Serial.read());
}



